==> vilistextum-2.3.2.tar.gz: vilistextum-v2.3.2/src/html.c <==
/*
 * vilistextum v2.3.2 21.05.01 - a html to text converter
 * Copyright (C) 1998-2001 Patric Mller
 * bhaak@gmx.net
 * http://www.diax.ch/users/bhaak/vilistextum/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 *
 *
 *  history
 *  19.04.2001: get_attr parses text of a alt attribute 
 *  20.04.2001: added references ala lynx
 *  
 *
 */

#include <ctype.h>
#include <stdio.h>
#include <string.h>

#include "main.h"
#include "html_tag.h"
#include "text.h"
#include "microsoft.h"
#include "latin1.h"
#include "fileio.h"
#include "debug.h"

int pre=0; // fuer PRE-Tag

char attr_name[DEF_STR_LEN], /* Attribut Name innerhalb eines HTML-TAGs */
	   attr_ctnt[DEF_STR_LEN]; /* Attribut Inhalt innerhalb eines HTML-TAGs */

// ------------------------------------------------

// get the next attribute writes it to attr_name and attr_ctnt.
// attr_name is converted to uppercase. 
int get_attr() // FIXME change to get_attr(char *name, char *ctnt)
{
  int i;
	char temp[DEF_STR_LEN];
#ifdef proc_debug 
  printf("get_attr()\n");
#endif

  attr_name[0] = '\0';
  attr_ctnt[0] = '\0';

  while ((isspace(ch)) && (ch!='>')) { ch=read_char(); }
  if (ch=='>') { return '>'; };

  // read attribute's name
  i=1;
  attr_name[0] = ch;

	while ((ch!='=') && (ch!='>'))
	{ ch=read_char(); attr_name[i++] = ch; }
  attr_name[i-1] = '\0';
	
  if (ch=='>') { attr_ctnt[0]='\0'; return '>'; }

  // content of attribute
	ch=read_char();
	// skip white_space
	while ((isspace(ch)) && (ch!='>')) { ch=read_char(); }
	temp[0] = '\0';

	// if quoted
  if ((ch=='"') || (ch=='\''))
  {
		// attribute looks like alt="bla" or alt='bla'
		// we'll have to remember what the quote was
		int quote=ch;
    i=0;
    ch=read_char();
    while(quote!=ch) { temp[i++] = ch; ch=read_char(); }
    temp[i] = '\0';
    ch=read_char();
  }
  else
  {
		// attribute looks like alt=bla
    i=1;
    temp[0] = ch;
    while ((ch!='>') && (!isspace(ch))) { ch=read_char(); temp[i++] = ch; }
    temp[i-1] = '\0';
  }

	//	printf("AttributInhalt: #%s#\n", temp);
	uppercase_str(attr_name);
	//printf(" %s %s\n", attr_name, temp);
  if (strcmp(attr_name, "ALT")==0) { parse_entities(temp); }
	//  printf("AttributInhalt: %s#-#\n", temp);
	strcpy(attr_ctnt, temp);
  //printf("Attribut: %s; Inhalt: %s#-#\n", attr_name, attr_ctnt);

#ifdef attr_debug
  printf("Attribut: %s; Inhalt: %s#-#\n", attr_name, attr_ctnt);
#endif
#ifdef proc_debug
  printf("get_attr() ende\n");
#endif
  return ch;
} /* end get_attr */

// ------------------------------------------------ 

void html()
{
  int i;
	char str[DEF_STR_LEN];
  //align[0]=LEFT;

  for (;;)
  {
    ch = read_char();
#ifdef default_debug
    printf("default: ch=%c ; %d\n",ch,ch);
#endif

    switch (ch)
    {
      case '<':
#ifdef debug
        printf("<: \n");
#endif
        html_tag();
        break;

			// Entities 
      case '&':
#ifdef debug
        printf("&: \n");
#endif
        i=1;
        str[0] = ch;
        do
        {
          ch = read_char();
          str[i++] = ch;
        }
        while ((isalnum(ch)) || (ch=='#'));

				// if last char is no ';', then the string is no valid entity, maybe
				// it is something like &nbsp or even '& '
				if (ch!=';') 
				{ 
					// no ';' at end
					str[i-1] = '\0'; 
					// save last char 
					goback_char(1); }  
				else 
				{					
					// valid entity
					str[i] = '\0';
					//					strcpy(tmpstr, str);
				}
#ifdef debug
        printf("check &: %s\n",str);
#endif
				parse_entity(str);
				wort_plus_string(str);
				/*
 				if ((entity_number(str)) || (latin1(str)) || (microsoft_entities(str))) 
				{
					// if true entity was known
					wort_plus_string(str);
				}
        else
        {
					if (errorlevel>=1) { fprintf(stderr, "entity %s unknown\n",tmpstr); }
					wort_plus_string(tmpstr);
					// FIXME MAYBE???
					//goback_char(strlen(str)-2);
					//wort_plus_ch(str[0]);
					//wort_plus_ch(str[1]);
				}
				*/
#ifdef debug
        printf("&: %s\n",str);
#endif
        break;

      case  9: // TAB */
      case 13: // CR */
#ifdef debug
        printf("TAB or CR\n");
#endif
      break;

      case '\n':
#ifdef debug
        printf("\\n: \n");
#endif
        wort_ende();
        if (pre) { line_break(); }
        break;

		  case EOF:
#ifdef debug
        printf("EOF: \n");
#endif
        quit();
        break;

      // Microsoft...
  		case 0x80: case 0x81: case 0x82: case 0x83: case 0x84: case 0x85: case 0x86: case 0x87: 
		  case 0x88: case 0x89: case 0x8a: case 0x8b: case 0x8c: case 0x8d:	case 0x8e: case 0x8f:
		  case 0x90: case 0x91: case 0x92: case 0x93: case 0x94: case 0x95: case 0x96: case 0x97:
      case 0x98: case 0x99: case 0x9a: case 0x9b: case 0x9c: case 0x9d: case 0x9e: case 0x9f:
				if (microsoft) { microsoft_character(ch); }
				else wort_plus_ch(ch);
				break;
		
      default:
#ifdef default_debug
        printf("default: ch=%c ; %d\n",ch,ch);
#endif
        if (pre==0)
        {
          if (ch==' ') { wort_ende(); }
          else { wort_plus_ch(ch); }
        }
        else { wort_plus_ch(ch); }
        break;
    }
  } /* next */
} /* end html */

// ------------------------------------------------

// used when there's only the align-attribut to be checked 
void check_for_center()
{
#ifdef proc_debug
  printf("check_for_center()\n");
#endif
  while (ch!='>')
  {
    ch=get_attr();
#ifdef debug
    printf(" Attr: %s; Inhalt: %s#\n",attr_name,attr_ctnt);
#endif
    if (strcmp("ALIGN", attr_name)==0)
    {
      uppercase_str(attr_ctnt);
      if (strcmp("LEFT",   attr_ctnt)==0) { push_align(LEFT);   }
      else if (strcmp("CENTER", attr_ctnt)==0) { push_align(CENTER); }
      else if (strcmp("RIGHT",  attr_ctnt)==0) { push_align(RIGHT);  }
      else if (strcmp("JUSTIFY", attr_ctnt)==0) { push_align(LEFT);  }		
      else { if (errorlevel>=2) { fprintf(stderr, "Kein LEFT|CENTER|RIGHT gefunden!\n"); push_align(LEFT); } }
    } else {
      /* kein ALIGN gefunden */
      push_align(LEFT);
    }
  }

#ifdef proc_debug
  printf("check_for_center() ende\n");
#endif
} /* end check_for_center */

// ------------------------------------------------

void start_p()
{
#ifdef proc_debug
  printf("\nstart_p()\n");
#endif
	push_align(LEFT);
  neuer_paragraph();
  check_for_center();
#ifdef proc_debug
  printf("start_p() ende\n");
#endif
} /* end start_p */

// ------------------------------------------------

void start_div(int a)
{
#ifdef proc_debug
  printf("start_div()\n"),
#endif
		//  paragraphen_ende();

	line_break();

  if (a!=0) { div_test=a; push_align(div_test); }
  else { check_for_center(); }
#ifdef proc_debug
  printf("start_div() ende\n");
#endif
} /* end start_div */

// ------------------------------------------------

void end_div()
{
#ifdef proc_debug
  printf("end_div()\n"),
#endif
  wort_ende();

  if (paragraph!=0) { paragraphen_ende(); }
  else { print_zeile(); }
  pop_align(); /* einer fr start_div */
  div_test = 0;
#ifdef proc_debug
  printf("end_div() ende\n");
#endif
} /* end_div */

// ------------------------------------------------

int references_count=0;
char references[DEF_STR_LEN];
char *schemes[] = {"ftp://","file://" ,"http://" ,"gopher://" ,"mailto:" ,"news:" ,"nntp://" ,"telnet://" ,"wais://" ,"prospero://" };

// handles <A href"..."></A>
void href()
{  
	char tmp[DEF_STR_LEN];

	if (!option_links) { return; }

	while (ch!='>')
  {
    ch=get_attr();
    if (strcmp("HREF", attr_name)==0)
		{
			if ((strstr(attr_ctnt, "://")!=NULL) || (strncmp(attr_ctnt, "mailto:", 7)==0) || (strncmp(attr_ctnt, "news:", 5)==0))
			{
				references_count++;
				//fprintf(stderr, " %d. %s\n", references_count, attr_ctnt);

				//fprintf(stderr, "");
				parse_entities(attr_ctnt);
				sprintf(tmp, "[%d]", references_count);
				wort_plus_string(tmp);
				
				sprintf(tmp, " %3d. %s\n", references_count, attr_ctnt);
				strcat(references, tmp);
				//fprintf(stderr, references);
			}
    }
  } 
} // end href

// ------------------------------------------------

void href_output()
{
	if (option_links)
	{
		if (references_count!=0)
		{
			output_string(" References:");
			output_string(references);
		}
	}
} // end href_output

// ------------------------------------------------

// find alt attribute in current tag
void image(char *alt_text, int show_alt)
{
  int found_alt=0;
#ifdef proc_debug
  printf("image()\n");
#endif
  while (ch!='>')
  {
    ch=get_attr();
    if (strcmp("ALT", attr_name)==0)
    {
			if (!(remove_empty_alt && (strcmp("", attr_ctnt)==0))) 
			{ 
				wort_plus_ch('['); wort_plus_string(attr_ctnt); wort_plus_ch(']');
			}
      found_alt=1;
    }
  }
  if ((found_alt==0) && (show_alt)) 
	{	wort_plus_ch('['); wort_plus_string(alt_text); wort_plus_ch(']'); }
#ifdef proc_debug
  printf("image() ende\n");
#endif
} /* end image */

// ------------------------------------------------

void friss_kommentar()
{
	int c, dontquit=1;
	while (dontquit) 
	{
		c=read_char();
		if (c=='-')
		{
		  c=read_char();
			while (c=='-') 
		  {
			  c=read_char();
				if (c=='>') { dontquit=0; }
			}
		}
	}
	goback_char(1);
} // end friss_kommentar

// ------------------------------------------------

==> vilistextum-2.4.1.tar.gz: vilistextum-2.4.1/src/html.c <==
/*
 * Copyright (C) 1998-2001 Patric Mller
 * bhaak@gmx.net
 * http://www.diax.ch/users/bhaak/vilistextum/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 *
 *
 *  history
 *  19.04.2001: get_attr parses text of a alt attribute 
 *  20.04.2001: added references ala lynx
 *  24.08.2001: Frisskommentar could'nt cope with <!--comment-->  
 *  03.09.2001: check_for_center worked only correct if align was last attribute
 *
 */

#include <ctype.h>
#include <stdio.h>
#include <string.h>

#include "main.h"
#include "html_tag.h"
#include "text.h"
#include "microsoft.h"
#include "latin1.h"
#include "fileio.h"
#include "debug.h"

int pre=0; // fuer PRE-Tag

char attr_name[DEF_STR_LEN], /* Attribut Name innerhalb eines HTML-TAGs */
	   attr_ctnt[DEF_STR_LEN]; /* Attribut Inhalt innerhalb eines HTML-TAGs */

// ------------------------------------------------

// get the next attribute writes it to attr_name and attr_ctnt.
// attr_name is converted to uppercase. 
int get_attr() // FIXME change to get_attr(char *name, char *ctnt)
{
  int i;
	char temp[DEF_STR_LEN];
#ifdef proc_debug 
  printf("get_attr()\n");
#endif

  attr_name[0] = '\0';
  attr_ctnt[0] = '\0';

  while ((isspace(ch)) && (ch!='>')) { ch=read_char(); }
  if (ch=='>') { return '>'; };

  // read attribute's name
  i=1;
  attr_name[0] = ch;

	while ((ch!='=') && (ch!='>'))
	{ ch=read_char(); attr_name[i++] = ch; }
  attr_name[i-1] = '\0';
	
  if (ch=='>') { attr_ctnt[0]='\0'; return '>'; }

  // content of attribute
	ch=read_char();
	// skip white_space
	while ((isspace(ch)) && (ch!='>')) { ch=read_char(); }
	temp[0] = '\0';

	// if quoted
  if ((ch=='"') || (ch=='\''))
  {
		// attribute looks like alt="bla" or alt='bla'
		// we'll have to remember what the quote was
		int quote=ch;
    i=0;
    ch=read_char();
    while(quote!=ch) { temp[i++] = ch; ch=read_char(); }
    temp[i] = '\0';
    ch=read_char();
  }
  else
  {
		// attribute looks like alt=bla
    i=1;
    temp[0] = ch;
    while ((ch!='>') && (!isspace(ch))) { ch=read_char(); temp[i++] = ch; }
    temp[i-1] = '\0';
  }

	//	printf("AttributInhalt: #%s#\n", temp);
	uppercase_str(attr_name);
	//printf(" %s %s\n", attr_name, temp);
  if (strcmp(attr_name, "ALT")==0) { parse_entities(temp); }
	//  printf("AttributInhalt: %s#-#\n", temp);
	strcpy(attr_ctnt, temp);
  //printf("Attribut: %s; Inhalt: %s#-#\n", attr_name, attr_ctnt);

#ifdef attr_debug
  printf("Attribut: %s; Inhalt: %s#-#\n", attr_name, attr_ctnt);
#endif
#ifdef proc_debug
  printf("get_attr() ende\n");
#endif
  return ch;
} /* end get_attr */

// ------------------------------------------------ 

void html()
{
  int i;
	char str[DEF_STR_LEN];
  //align[0]=LEFT;

  for (;;)
  {
    ch = read_char();
#ifdef default_debug
    printf("default: ch=%c ; %d\n",ch,ch);
#endif

    switch (ch)
    {
      case '<':
#ifdef debug
        printf("<: \n");
#endif
        html_tag();
        break;

			// Entities 
      case '&':
#ifdef debug
        printf("&: \n");
#endif
        i=1;
        str[0] = ch;
        do
        {
          ch = read_char();
          str[i++] = ch;
        }
        while ((isalnum(ch)) || (ch=='#'));

				// if last char is no ';', then the string is no valid entity, maybe
				// it is something like &nbsp or even '& '
				if (ch!=';') 
				{ 
					// no ';' at end
					str[i-1] = '\0'; 
					// save last char 
					goback_char(1); }  
				else 
				{					
					// valid entity
					str[i] = '\0';
					//					strcpy(tmpstr, str);
				}
#ifdef debug
        printf("check &: %s\n",str);
#endif
				parse_entity(str);
				wort_plus_string(str);
				/*
 				if ((entity_number(str)) || (latin1(str)) || (microsoft_entities(str))) 
				{
					// if true entity was known
					wort_plus_string(str);
				}
        else
        {
					if (errorlevel>=1) { fprintf(stderr, "entity %s unknown\n",tmpstr); }
					wort_plus_string(tmpstr);
					// FIXME MAYBE???
					//goback_char(strlen(str)-2);
					//wort_plus_ch(str[0]);
					//wort_plus_ch(str[1]);
				}
				*/
#ifdef debug
        printf("&: %s\n",str);
#endif
        break;

      case  9: // TAB */
      case 13: // CR */
#ifdef debug
        printf("TAB or CR\n");
#endif
      break;

      case '\n':
#ifdef debug
        printf("\\n: \n");
#endif
        wort_ende();
        if (pre) { line_break(); }
        break;

		  case EOF:
#ifdef debug
        printf("EOF: \n");
#endif
        quit();
        break;

      // Microsoft...
  		case 0x80: case 0x81: case 0x82: case 0x83: case 0x84: case 0x85: case 0x86: case 0x87: 
		  case 0x88: case 0x89: case 0x8a: case 0x8b: case 0x8c: case 0x8d:	case 0x8e: case 0x8f:
		  case 0x90: case 0x91: case 0x92: case 0x93: case 0x94: case 0x95: case 0x96: case 0x97:
      case 0x98: case 0x99: case 0x9a: case 0x9b: case 0x9c: case 0x9d: case 0x9e: case 0x9f:
				if (microsoft) { microsoft_character(ch); }
				else wort_plus_ch(ch);
				break;
		
      default:
#ifdef default_debug
        printf("default: ch=%c ; %d\n",ch,ch);
#endif
        if (pre==0)
        {
          if (ch==' ') { wort_ende(); }
          else { wort_plus_ch(ch); }
        }
        else { wort_plus_ch(ch); }
        break;
    }
  } /* next */
} /* end html */

// ------------------------------------------------

// used when there's only the align-attribut to be checked 
void check_for_center()
{
	int found=0;
#ifdef proc_debug
  printf("check_for_center()\n");
#endif
  while (ch!='>')
  {
    ch=get_attr();
#ifdef debug
    printf(" Attr: %s; Inhalt: %s#\n",attr_name,attr_ctnt);
#endif
    if (strcmp("ALIGN", attr_name)==0)
    {
      uppercase_str(attr_ctnt);
      if (strcmp("LEFT",   attr_ctnt)==0) { push_align(LEFT); found=1;  }
      else if (strcmp("CENTER", attr_ctnt)==0) { push_align(CENTER); found=1; }
      else if (strcmp("RIGHT",  attr_ctnt)==0) { push_align(RIGHT); found=1;  }
      else if (strcmp("JUSTIFY", attr_ctnt)==0) { push_align(LEFT); found=1;  }		
      else { if (errorlevel>=2) { fprintf(stderr, "Kein LEFT|CENTER|RIGHT gefunden!\n"); push_align(LEFT); } }
    } else {
      /* kein ALIGN gefunden */
//      push_align(LEFT);
    }
  } 
 
  // kein ALIGN gefunden
  // sollte das nicht hier stehen? mit found==0 gibt es seltsamste stoerungen
  //if (found=0) { push_align(LEFT); }
	
#ifdef proc_debug
  printf("check_for_center() ende\n");
#endif
} /* end check_for_center */

// ------------------------------------------------

void start_p()
{
#ifdef proc_debug
  printf("\nstart_p()\n");
#endif
  push_align(LEFT);
  neuer_paragraph();
  check_for_center();
#ifdef proc_debug
  printf("start_p() ende\n");
#endif
} /* end start_p */

// ------------------------------------------------

void start_div(int a)
{
#ifdef proc_debug
  printf("start_div()\n"),
#endif
		//  paragraphen_ende();

	line_break();

  if (a!=0) { div_test=a; push_align(div_test); }
  else { check_for_center(); }
#ifdef proc_debug
  printf("start_div() ende\n");
#endif
} /* end start_div */

// ------------------------------------------------

void end_div()
{
#ifdef proc_debug
  printf("end_div()\n"),
#endif
  wort_ende();

  if (paragraph!=0) { paragraphen_ende(); }
  else { print_zeile(); }
  pop_align(); /* einer fr start_div */
  div_test = 0;
#ifdef proc_debug
  printf("end_div() ende\n");
#endif
} /* end_div */

// ------------------------------------------------

int references_count=0;
char references[DEF_STR_LEN];
char *schemes[] = {"ftp://","file://" ,"http://" ,"gopher://" ,"mailto:" ,"news:" ,"nntp://" ,"telnet://" ,"wais://" ,"prospero://" };

// handles <A href"..."></A>
void href()
{  
	char tmp[DEF_STR_LEN];

	if (!option_links) { return; }

	while (ch!='>')
  {
    ch=get_attr();
    if (strcmp("HREF", attr_name)==0)
		{
			if ((strstr(attr_ctnt, "://")!=NULL) || (strncmp(attr_ctnt, "mailto:", 7)==0) || (strncmp(attr_ctnt, "news:", 5)==0))
			{
				references_count++;
				//fprintf(stderr, " %d. %s\n", references_count, attr_ctnt);

				//fprintf(stderr, "");
				parse_entities(attr_ctnt);
				sprintf(tmp, "[%d]", references_count);
				wort_plus_string(tmp);
				
				sprintf(tmp, " %3d. %s\n", references_count, attr_ctnt);
				strcat(references, tmp);
				//fprintf(stderr, references);
			}
    }
  } 
} // end href

// ------------------------------------------------

void href_output()
{
	if (option_links)
	{
		if (references_count!=0)
		{
			output_string("\n References:");
			output_string(references);
		}
	}
} // end href_output

// ------------------------------------------------

// find alt attribute in current tag
void image(char *alt_text, int show_alt)
{
  int found_alt=0;
#ifdef proc_debug
  printf("image()\n");
#endif
  while (ch!='>')
  {
    ch=get_attr();
    if (strcmp("ALT", attr_name)==0)
    {
			if (!(remove_empty_alt && (strcmp("", attr_ctnt)==0))) 
			{ 
				wort_plus_ch('['); wort_plus_string(attr_ctnt); wort_plus_ch(']');
			}
      found_alt=1;
    }
  }
  if ((found_alt==0) && (show_alt)) 
	{	wort_plus_ch('['); wort_plus_string(alt_text); wort_plus_ch(']'); }
#ifdef proc_debug
  printf("image() ende\n");
#endif
} /* end image */

// ------------------------------------------------

// simple finite state machine to eat up complete comment '!--'
void friss_kommentar()
{
	int c, dontquit=1;
#ifdef proc_debug
	printf("Frisskommentar start");
#endif
	while (dontquit) 
	{
		c=read_char();
#ifdef debug
		printf("%c", c);
#endif
		if (c=='-')
		{
		  c=read_char();
#ifdef debug
			printf("%c", c);
#endif
			while (c=='-') 
		  {
			  c=read_char();
#ifdef debug
				printf("%c", c);
#endif
				if (c=='>') { dontquit=0; }
			}
		}
	}

#ifdef debug
	printf("\n");
#endif
#ifdef proc_debug
	printf("Frisskommentar ende");
#endif
} // end friss_kommentar

// ------------------------------------------------
